package com.supermap.samplecode.realspace;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import com.supermap.data.Workspace;
import com.supermap.ui.SceneControl;

/**
 * <p>
 * Title:剖面分析
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap iObjects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：如何基于地形和模型进行剖面分析
 * 2、示例数据：安装目录\SampleData\Chart\Chart.smwu;
 * 3、关键类型/成员: 
 *  	Workspace.open 方法
 *  	Workspace.getDatasources 方法
 *  	TerrainLayers.add 方法
 *  	Layers.add 方法
 *  	Scene.pixelToGlobe 方法
 *  	Scene.setCamera 方法
 *  	ProfileAnalyst.calculatePlumbProfile 方法
 *  	DatasetVector.getRecordset 方法
 *  	Recordset.BatchEditor.setMaxRecordCount 方法
 *  	Recordset.BatchEditor.begin 方法
 *  	Recordset.BatchEditor.update 方法
 *  	Datasets.create 方法
 *  	GeoLine.addPart 方法
 *  	GeoLine3D.addPart 方法
 *  	GeoLine.setStyle 方法
 *  	GeoLine3D.setStyle3D 方法
 *  	GeoText.setTextStyle 方法
 *  	TrackingLayer.add 方法
 *  	SceneControl.MouseEvent 事件
 * 4、使用步骤：
 *   (1) 运行应用程序
 *   (2) 待场景数据加载完，点击“绘制切割线”按钮，在场景中绘制切割线
 *   (3) 绘制切割线后，点击“分析”按钮，进行剖面分析。分析完成后弹出绘制有剖面图的窗体
 *   (4) 点击"重置相机"按钮，可以将场景定位到模型所在的范围
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */

/**
 * <p>
 * Title: Profile analysis
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Copyright Statement----------------------------
 * SuperMap Objects Java Sample Code
 * Copyright: SuperMap Software Co., Ltd. All Rights Reserved.
 * ------------------------------------------------------------------------------
 *-----------------------Description--------------------------
 * 1, Introduction: Perform profile analysis based on terrain and model data
 * 2, Sample data: Installation directory\SampleData\BeijingDEM\BeijingDEM.smwu;
 * 3, Key class/method: 
 *  	Workspace.open method
 *  	Workspace.getDatasources method
 *  	TerrainLayers.add method
 *  	Layers.add method
 *  	Scene.pixelToGlobe method
 *  	Scene.setCamera method
 *  	ProfileAnalyst.calculatePlumbProfile method
 *  	DatasetVector.getRecordset method
 *  	Recordset.BatchEditor.setMaxRecordCount method
 *  	Recordset.BatchEditor.begin method
 *  	Recordset.BatchEditor.update method
 *  	Datasets.create method
 *  	GeoLine.addPart method
 *  	GeoLine3D.addPart method
 *  	GeoLine.setStyle method
 *  	GeoLine3D.setStyle3D method
 *  	GeoText.setTextStyle method
 *  	TrackingLayer.add method
 *  	SceneControl.MouseEvent event
 * 4, Step:
 *   (1) Run application
 *   (2) After the data is loaded, click "Draw line" to draw a line
 *   (3) After drawing, click "Analyze" to perform profile analysis
 *   (4) Click "Reset" to relocate the scene
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */

public class MainFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private JPanel m_jContentPane;
	
	private JToolBar m_jToolBar;
	
	private JButton m_drawLineButton;
	
	private JButton m_analystButton;
	
	private JButton m_resetCameraButton;
	
	private Workspace m_workspace;
	
	private SceneControl m_sceneControl;
	
	private SampleRun m_sampleRun;
	
	/**
	 * 程序入口
	 * Start
	 */
	public static void main(String args[]) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainFrame frame = new MainFrame();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * 构造方法
	 * Construct method
	 */
	public MainFrame() {
		super();
		initialize();
	}
	
	/**
	 * 初始化
	 * Initialization
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
		this.setTitle(getText("title"));
		this.setExtendedState(Frame.NORMAL);
		this.setSize(800,500);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.addWindowListener(new WindowAdapter(){
			public void windowOpened(WindowEvent e){
				m_workspace=new Workspace();
				// 在主窗体加载时，初始化SampleRun类型，来完成功能的展现
				// Use SampleRun to realize the function
				m_sampleRun=new SampleRun(m_workspace,m_sceneControl,m_analystButton);
			}
			// 在窗体关闭时，释放相关的资源
			// Destroy resource
			public void windowClosed(WindowEvent e){
				m_sceneControl.dispose();
				m_workspace.dispose();
			}
		});
	}

	/**
	 * 创建主面板
	 * Create main frame
	 */
	private JPanel getJContentPane() {
		if(m_jContentPane==null) {
			m_jContentPane=new JPanel();
			m_jContentPane.setLayout(new BorderLayout());
			m_jContentPane.add(getToolBar(),BorderLayout.NORTH);
			m_jContentPane.add(getSceneControl(),BorderLayout.CENTER);
		}
		return m_jContentPane;
	}

	/**
	 * 创建工具条
	 * Create toolbar
	 * 
	 */
	private JToolBar getToolBar() {
		if(m_jToolBar==null){
			m_jToolBar=new JToolBar();
			m_jToolBar.add(getDrawLineButton());
			m_jToolBar.add(getAnalystButton());
			m_jToolBar.add(getResetCameraButton());
		}
		return m_jToolBar;
	}

	/**
	 * 创建“绘制切割线”按钮并为其添加监听事件
	 * Create “Draw line” button and add event
	 */
	private JButton getDrawLineButton() {
		if(m_drawLineButton==null){
			m_drawLineButton=new JButton();
			m_drawLineButton.setText(getText("m_drawLineButton"));
			m_drawLineButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					m_sampleRun.startDraw();
				}
			});
		}
		return m_drawLineButton;
	}
	
	/**
	 * 创建“分析”按钮并为其添加监听事件
	 * Create "Analyze" button and add event
	 */
	private JButton getAnalystButton() {
		if(m_analystButton==null){
			m_analystButton=new JButton();
			m_analystButton.setText(getText("m_analystButton"));
			m_analystButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					m_sampleRun.analyst();
				}
			});
		}
		return m_analystButton;
	}

	/**
	 * 创建“重置相机”按钮并为其添加监听事件
	 * Add "Reset" button and add event
	 */
	private JButton getResetCameraButton() {
		if(m_resetCameraButton==null){
			m_resetCameraButton=new JButton();
			m_resetCameraButton.setText(getText("m_resetCameraButton"));
			m_resetCameraButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					m_sampleRun.setCamera();
				}
			});
		}
		return m_resetCameraButton;
	}

	/**
	 * 创建三维场景控件
	 * Create 3D scene control
	 */
	private SceneControl getSceneControl() {
		if(m_sceneControl==null){
			m_sceneControl=new SceneControl();
		}
		return m_sceneControl;
	}
	
	private String getText(String name)
	{
		String text = "";
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			if(name == "title")
			{
				text = "剖面分析";
			}
			if(name == "m_drawLineButton")
			{
				text = "绘制切割线";
			}
			if(name == "m_analystButton")
			{
				text = "分析";
			}
			if(name == "m_resetCameraButton")
			{
				text = "重置相机";
			}			
		}
		else
		{
			if(name == "title")
			{
				text = "Profile analysis";
			}
			if(name == "m_drawLineButton")
			{
				text = "Draw line";
			}
			if(name == "m_analystButton")
			{
				text = "Analyze";
			}
			if(name == "m_resetCameraButton")
			{
				text = "Reset";
			}
		}
		return text;
	}
}
