package com.supermap.samplecode.realspace;

import java.awt.Color;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import javax.swing.JButton;
import com.supermap.data.AltitudeMode;
import com.supermap.data.DatasetGrid;
import com.supermap.data.DatasetVector;
import com.supermap.data.Datasource;
import com.supermap.data.GeoLine3D;
import com.supermap.data.GeoStyle3D;
import com.supermap.data.Point2D;
import com.supermap.data.Point3D;
import com.supermap.data.Point3Ds;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.realspace.Camera;
import com.supermap.realspace.Layer3DSettingGrid;
import com.supermap.realspace.Layer3DSettingVector;
import com.supermap.realspace.PixelToGlobeMode;
import com.supermap.realspace.spatialanalyst.ProfileAnalyst;
import com.supermap.ui.SceneControl;
import com.supermap.ui.MapControl.Cursors;

/**
 * <p>
 * Title:剖面分析
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap iObjects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：如何基于地形和模型进行剖面分析
 * 2、示例数据：安装目录\SampleData\Chart\Chart.smwu;
 * 3、关键类型/成员: 
 *  	Workspace.open 方法
 *  	Workspace.getDatasources 方法
 *  	TerrainLayers.add 方法
 *  	Layers.add 方法
 *  	Scene.pixelToGlobe 方法
 *  	Scene.setCamera 方法
 *  	ProfileAnalyst.calculatePlumbProfile 方法
 *  	DatasetVector.getRecordset 方法
 *  	Recordset.BatchEditor.setMaxRecordCount 方法
 *  	Recordset.BatchEditor.begin 方法
 *  	Recordset.BatchEditor.update 方法
 *  	Datasets.create 方法
 *  	GeoLine.addPart 方法
 *  	GeoLine3D.addPart 方法
 *  	GeoLine.setStyle 方法
 *  	GeoLine3D.setStyle3D 方法
 *  	GeoText.setTextStyle 方法
 *  	TrackingLayer.add 方法
 *  	SceneControl.MouseEvent 事件
 * 4、使用步骤：
 *   (1) 运行应用程序
 *   (2) 待场景数据加载完，点击“绘制切割线”按钮，在场景中绘制切割线
 *   (3) 绘制切割线后，点击“分析”按钮，进行剖面分析。分析完成后弹出绘制有剖面图的窗体
 *   (4) 点击"重置相机"按钮，可以将场景定位到模型所在的范围
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */

/**
 * <p>
 * Title: Profile analysis
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Copyright Statement----------------------------
 * SuperMap Objects Java Sample Code
 * Copyright: SuperMap Software Co., Ltd. All Rights Reserved.
 * ------------------------------------------------------------------------------
 *-----------------------Description--------------------------
 * 1, Introduction: Perform profile analysis based on terrain and model data
 * 2, Sample data: Installation directory\SampleData\BeijingDEM\BeijingDEM.smwu;
 * 3, Key class/method: 
 *  	Workspace.open method
 *  	Workspace.getDatasources method
 *  	TerrainLayers.add method
 *  	Layers.add method
 *  	Scene.pixelToGlobe method
 *  	Scene.setCamera method
 *  	ProfileAnalyst.calculatePlumbProfile method
 *  	DatasetVector.getRecordset method
 *  	Recordset.BatchEditor.setMaxRecordCount method
 *  	Recordset.BatchEditor.begin method
 *  	Recordset.BatchEditor.update method
 *  	Datasets.create method
 *  	GeoLine.addPart method
 *  	GeoLine3D.addPart method
 *  	GeoLine.setStyle method
 *  	GeoLine3D.setStyle3D method
 *  	GeoText.setTextStyle method
 *  	TrackingLayer.add method
 *  	SceneControl.MouseEvent event
 * 4, Step:
 *   (1) Run application
 *   (2) After the data is loaded, click "Draw line" to draw a line
 *   (3) After drawing, click "Analyze" to perform profile analysis
 *   (4) Click "Reset" to relocate the scene
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */

public class SampleRun {

	private Workspace m_workspace;
	
	private SceneControl m_sceneControl;
	
	private JButton m_analystButton;
	
	private Datasource m_datasource;
	
	private DatasetGrid m_surface;
	
	private DatasetVector[] m_models;

	private boolean m_canDrawLine;
	
	private Point2D m_startPoint;
	
	private Point2D m_endPoint;
	
	private int m_count;
	
	private ResultDialog m_resultDialog;
	
	/**
	 * 构造方法
	 * Construct method
	 */
	public SampleRun(Workspace workspace,SceneControl sceneControl,JButton analystButton){
		m_workspace=workspace;
		m_sceneControl=sceneControl;
		m_sceneControl.getScene().setWorkspace(workspace);
		m_analystButton=analystButton;
		
		initialize();
	}

	/**
	 * 打开工作空间，并加载数据到场景；注册事件
	 * Load workspace, load data, add listener
	 */
	private void initialize() {
		try	{
			// 打开工作空间，获取数据源
			// Open workspace and datasource
			WorkspaceConnectionInfo info = new WorkspaceConnectionInfo("../../SampleData/BeijingDEM/BeijingDEM.smwu");
			m_workspace.open(info);
			m_datasource = m_workspace.getDatasources().get("BeijingDEM");

			// 获取地形和模型数据，并加载到当前场景中
			// Get terrain and model data to add them to the scene
			m_surface =(DatasetGrid) m_datasource.getDatasets().get("DEM25");
			DatasetVector models =(DatasetVector) m_datasource.getDatasets().get("Buildings");
			m_models = new DatasetVector[] { models };
			m_sceneControl.getScene().getTerrainLayers().add(m_surface, true);
			m_sceneControl.getScene().getLayers().add(m_surface, new Layer3DSettingGrid(), true);
			m_sceneControl.getScene().getLayers().add(models, new Layer3DSettingVector(), true);

			// 调整相机
			// Set camera
			setCamera();
			m_sceneControl.getScene().refresh();

			// 添加事件
			// Add event
			mapControlAddListener();
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
		
	}
	
	/**
	 * 开始绘制
	 * Start drawing
	 */
	public void startDraw()
	{
		try	{
			m_count = 0;
			m_startPoint = new Point2D();
			m_endPoint = new Point2D();
			m_sceneControl.getScene().getTrackingLayer().clear();
			m_sceneControl.getScene().refresh();
			m_canDrawLine = true;
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
	}

	/**
	 * 鼠标监听事件
	 * Mouse listener event
	 */
	private void mapControlAddListener() {
		m_sceneControl.addMouseListener(new MouseAdapter(){
			
			public void mouseClicked(MouseEvent e){
				try {
					// 单击左键
					// Click
					if (e.getButton() == 1 && m_canDrawLine) {
						m_count++;
						Point point = new Point(e.getX(), e.getY());
						Point3D globePoint = m_sceneControl.getScene().pixelToGlobe(point, PixelToGlobeMode.TERRAIN);

						if (globePoint != null)	{
							if (m_count % 2 == 1) {
								m_startPoint.setX(globePoint.getX());
								m_startPoint.setY(globePoint.getY());
								m_analystButton.setEnabled(false);
							} else {
								m_endPoint.setX(globePoint.getX());
								m_endPoint.setY(globePoint.getY());
								m_analystButton.setEnabled(true);
							}
						}
					}
				} catch (Exception ex) {
					System.out.print(ex.getMessage());
				}
			}
		});
		
		m_sceneControl.addMouseMotionListener(new MouseMotionAdapter() {
		
			public void mouseMoved(MouseEvent e){
				try {
					if (m_canDrawLine) {
						m_sceneControl.setCursor(Cursors.getCross());
						if (m_startPoint != null && m_count > 0 && m_count % 2 == 1) {
							Point point = new Point(e.getX(),e.getY());
							Point3D currentPoint = m_sceneControl.getScene().pixelToGlobe(point);
							Point3D fromPoint = new Point3D(m_startPoint.getX(), m_startPoint.getY(), 0);
							drawLine(fromPoint, currentPoint);
						}
					}
				} catch (Exception ex) {
					System.out.print(ex.getMessage());
				}
			}
		});
	}
	
	/**
	 * 进行剖面分析，生成结果剖面线，并绘制二维剖面图
	 * Perform profile analysis and build result showed in graph
	 */
	public void analyst() {
		try
		{
			if (m_startPoint != null && m_endPoint != null && m_startPoint != m_endPoint)
			{
				m_canDrawLine = false;
				if (m_models.length > 0 && m_surface != null) {
					String resultName = "ProfileResult3D";
					if (m_datasource.getDatasets().contains(resultName)) {
						m_datasource.getDatasets().delete(resultName);
					}
					DatasetVector resultDatset = ProfileAnalyst.calculatePlumbProfile(m_startPoint, m_endPoint, m_models, m_surface, m_datasource, resultName);
					if (resultDatset != null) {
						if (m_resultDialog != null)	{
							if (!m_resultDialog.isShowing()) {
								m_resultDialog = new ResultDialog(m_workspace, resultDatset, m_startPoint, m_endPoint, m_datasource);
							} else {
								m_resultDialog.refresh(resultDatset, m_startPoint, m_endPoint);
							}
						} else {
							m_resultDialog = new ResultDialog(m_workspace, resultDatset, m_startPoint, m_endPoint, m_datasource);
						}
					}
				}
			}
		} catch (Exception ex) {
			System.out.print(ex.getMessage());
		}
	}
	
	/**
	 * 在跟踪层上绘制切割线
	 * Add line in tracking layer
	 */
	public void drawLine(Point3D fromPoint, Point3D toPoint) {
		try {
			Point3Ds points = new Point3Ds();
			points.add(fromPoint);
			points.add(toPoint);
			GeoLine3D line = new GeoLine3D();
			line.addPart(points);
			GeoStyle3D style = new GeoStyle3D();
			style.setLineColor(Color.BLACK);
			style.setLineWidth(3);
			line.setStyle3D(style);
			m_sceneControl.getScene().getTrackingLayer().clear();
			m_sceneControl.getScene().getTrackingLayer().add(line, "line");
			m_sceneControl.getScene().refresh();
		}catch (Exception ex){
			System.out.print(ex.getMessage());
		}
	}

	/**
	 * 设置相机
	 * Set camera
	 */
	public void setCamera()
	{
		try	{
			Camera camera = new Camera();
			camera.setLongitude(116.127808232291);
			camera.setLatitude(40.6695126738027);
			camera.setAltitude(910.531396129169);
			camera.setAltitudeMode(AltitudeMode.CLAMP_TO_GROUND);
			camera.setTilt(74.5442362358473);
			camera.setHeading(322.506790889528);
			m_sceneControl.getScene().setCamera(camera);
			m_sceneControl.getScene().refresh();
		} catch (Exception ex){
			System.out.print(ex.getMessage());
		}
	}
	
	
	
	
}
